<?php
/**
 * Voicer converts text into human-like speech in more than 200 voices across 30+ languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/voicer
 *
 * @encoding        UTF-8
 * @version         2.1.3
 * @copyright       Copyright (C) 2018 - 2020 Merkulove (https://merkulov.design/). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Voicer;

use Merkulove\Voicer;
use Google\ApiCore\ApiException;
use Google\Cloud\TextToSpeech\V1\TextToSpeechClient;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to implement plugin settings.
 *
 * @since 2.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class Settings {

	/**
	 * Voicer Plugin settings.
	 *
	 * @var array()
	 * @since 2.0.0
	 **/
	public $options = [];

	/**
	 * The one true Settings.
	 *
	 * @var Settings
	 * @since 2.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Settings instance.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Get plugin settings. */
		$this->get_options();

	}

	/**
	 * Render settings tabs.
	 *
	 * @param array $tabs       - Array of available tabs.
	 * @param string $current   - Slug of active tab.
	 *
	 * @access private
	 * @since  2.1.0
	 *
	 * @return void
	 **/
	private function render_tabs( $tabs, $current ) {

		?>
        <hr class="mdc-plugin-menu">
        <hr class="mdc-list-divider">
        <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Plugin settings', 'voicer' ) ?></h6>
		<?php

		/** Plugin settings tabs. */
		foreach ( $tabs as $tab => $value ) {

			/** Prepare CSS classes. */
			$classes = [];
			$classes[] = 'mdc-list-item';

			/** Mark Active Tab. */
			if ( $tab === $current ) {
				$classes[] = 'mdc-list-item--activated';
			}

			/** Prepare link. */
			$link = '?page=mdp_voicer_settings&tab=' . $tab;

			?>
            <a class="<?php esc_attr_e( implode( ' ', $classes ) ); ?>" href="<?php esc_attr_e( $link ); ?>">
                <i class='material-icons mdc-list-item__graphic' aria-hidden='true'><?php esc_html_e( $value['icon'] ); ?></i>
                <span class='mdc-list-item__text'><?php esc_html_e( $value['name'] ); ?></span>
            </a>
			<?php
		}

	}

	/**
	 * Displays useful links for an activated and non-activated plugin.
	 *
	 * @since 2.0.0
     *
     * @return void
	 **/
	public function support_link() { ?>

        <hr class="mdc-list-divider">
        <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Helpful links', 'voicer' ) ?></h6>

        <a class="mdc-list-item" href="https://docs.merkulov.design/tag/voicer/" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true">collections_bookmark</i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'Documentation', 'voicer' ) ?></span>
        </a>

		<?php if ( PluginActivation::get_instance()->is_activated() ) : /** Activated. */ ?>
            <a class="mdc-list-item" href="https://1.envato.market/voicer-support" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">mail</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Get help', 'voicer' ) ?></span>
            </a>
            <a class="mdc-list-item" href="https://1.envato.market/cc-downloads" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">thumb_up</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Rate this plugin', 'voicer' ) ?></span>
            </a>
		<?php endif; ?>

        <a class="mdc-list-item" href="https://1.envato.market/cc-merkulove" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true">store</i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'More plugins', 'voicer' ) ?></span>
        </a>
		<?php

	}

	/** @noinspection PhpUnused */
	/**
	 * Render settings audio profile field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_audio_profile() {

	    /** Prepare options for select. */
		$options = [
			'wearable-class-device' => esc_html__( 'Smart watches and other wearables', 'voicer' ),
			'handset-class-device' => esc_html__( 'Smartphones', 'voicer' ),
			'headphone-class-device' => esc_html__( 'Earbuds or headphones', 'voicer' ),
			'small-bluetooth-speaker-class-device' => esc_html__( 'Small home speakers', 'voicer' ),
			'medium-bluetooth-speaker-class-device' => esc_html__( 'Smart home speakers', 'voicer' ),
			'large-home-entertainment-class-device' => esc_html__( 'Home entertainment systems', 'voicer' ),
			'large-automotive-class-device' => esc_html__( 'Car speakers', 'voicer' ),
			'telephony-class-application' => esc_html__( 'Interactive Voice Response', 'voicer' ),
		];

		/** Render select. */
		UI::get_instance()->render_select(
			$options,
			$this->options['audio-profile'], // Selected option.
			esc_html__( 'Audio Profile', 'voicer' ),
			esc_html__( 'Optimize the synthetic speech for playback on different types of hardware.', 'voicer' ),
			['name' => 'mdp_voicer_settings[audio-profile]']
		);
    }

	/** @noinspection PhpUnused */
	/**
	 * Render Speaking Rate/Speed field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_speaking_rate() {

		/** Render slider. */
		UI::get_instance()->render_slider(
			$this->options['speaking-rate'],
			0.25,
			4.0,
			0.25,
			esc_html__( 'Speaking Rate/Speed', 'voicer'),
			esc_html__( 'Speaking rate:', 'voicer') . ' <strong>' . esc_html( $this->options['speaking-rate'] ) . '</strong><br>',
			[
				'name' => 'mdp_voicer_settings[speaking-rate]',
				'class' => 'mdc-slider-width',
				'id' => 'mdp_voicer_settings_rate'
			],
			false
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Pitch field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_pitch() {

		/** Render slider. */
		UI::get_instance()->render_slider(
			$this->options['pitch'],
			-20,
			20,
			0.1,
			esc_html__( 'Pitch', 'voicer' ),
			esc_html__( 'Current pitch:', 'voicer') . ' <strong>' . esc_html( $this->options['pitch'] ) . '</strong>',
			[
				'name' => 'mdp_voicer_settings[pitch]',
				'class' => 'mdc-slider-width',
				'id' => 'mdp_voicer_settings_pitch'
			],
			false
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render settings current_language field.
	 *
	 * @throws ApiException
	 
	 * @since 2.0.0
	 * @access public
     *         
     * @return void        
	 **/
	public function render_language() {

		/** Setting custom exception handler. */
		set_exception_handler( [ ErrorHandler::class, 'exception_handler' ] );

		/** Create client object. */
		$client = new TextToSpeechClient();

		/** Perform list voices request. */
		$response = $client->listVoices();
		$voices   = $response->getVoices();

		if ( count( $voices ) === 0 ) {
			?>
            <div class="mdp-key-error"><?php esc_html_e( 'Failed to get the list of languages. The request failed. It looks like a problem with your API Key File. Make sure that you are using the correct key file, and that the quotas have not been exceeded. If you set security restrictions on a key, make sure that the current domain is added to the exceptions.', 'voicer' ); ?></div><?php
			return;
		}

		/** Prepare Languages Options. */
		$options = [];
		$options[] = esc_html__( 'Select Language', 'voicer' );
        foreach ( $voices as $voice ) {
        	$lang = $this->get_lang_by_code( $voice->getLanguageCodes() );
	        $options[$lang] = $lang;
        }

		/** Render Language select. */
		UI::get_instance()->render_select(
			$options,
			'',
			esc_html__('Language', 'voicer' ),
			'',
			[
                'name' => 'mdp_voicer_language_filter',
				'id' => 'mdp-voicer-language-filter'
            ]
		);

		?>

        <div class="mdc-text-field-helper-line mdp-voicer-helper-padding">
            <div class="mdc-text-field-helper-text mdc-text-field-helper-text--persistent"><?php esc_html_e( 'The list includes both standard and', 'voicer' ); ?>
                <a href="https://cloud.google.com/text-to-speech/docs/wavenet"
                   target="_blank"><?php esc_html_e( 'WaveNet voices', 'voicer' ); ?></a>.
				<?php esc_html_e( 'WaveNet voices are higher quality voices with different', 'voicer' ); ?>
                <a href="https://cloud.google.com/text-to-speech/pricing"
                   target="_blank"><?php esc_html_e( 'pricing', 'voicer' ); ?></a>;
				<?php esc_html_e( 'in the list, they have the voice type "WaveNet".', 'voicer' ); ?>
            </div>
        </div>

        <table id="mdp-voicer-settings-language-tbl" class="display stripe hidden">
            <thead>
            <tr>
                <th><?php esc_html_e( 'Language', 'voicer' ); ?></th>
                <th><?php esc_html_e( 'Voice', 'voicer' ); ?></th>
                <th><?php esc_html_e( 'Gender', 'voicer' ); ?></th>
            </tr>
            </thead>
            <tbody>
			<?php foreach ( $voices as $voice ) : ?>
                <tr <?php if ( $voice->getName() === $this->options['language'] ) { echo 'class="selected"'; } ?>>
                    <td class="mdp-lang-name">
						<?php echo esc_html( $this->get_lang_by_code( $voice->getLanguageCodes() ) ); // Language. ?>
                    </td>
                    <td>
                        <span class="mdp-lang-code" title="<?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code ?>"><?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code  ?></span>
                        -
                        <span><?php echo $this->get_voice_type( $voice->getName() ); //Voice Type ?></span>
                        -
                        <span class="mdp-voice-name" title="<?php echo esc_html( $voice->getName() ); // Voice name ?>"><?php echo esc_html( substr( $voice->getName(), -1 ) ); // Voice Variant ?></span>
                    </td>
                    <td>
						<?php
						/** SSML voice gender values from TextToSpeech\V1\SsmlVoiceGender. */
						$ssmlVoiceGender = [ 'SSML_VOICE_GENDER_UNSPECIFIED', 'Male', 'Female', 'Neutral' ];

						echo '<span title="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'voicer' ) . '"><img src="' . Voicer::$url . 'images/' . strtolower( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '.svg" alt="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'voicer' ) . '">' . esc_html( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '</span>'; ?>
                    </td>
                </tr>
			<?php endforeach;

			$client->close();

			?>
            </tbody>

        </table>

        <input id="mdp-voicer-settings-language" type='hidden' name='mdp_voicer_settings[language]'
               value='<?php echo esc_attr( $this->options['language'] ); ?>'>
        <input id="mdp-voicer-settings-language-code" type='hidden' name='mdp_voicer_settings[language-code]'
               value='<?php echo esc_attr( $this->options['language-code'] ); ?>'>
		<?php

		/** Restore previous exception handler. */
		restore_exception_handler();

    }

	/**
	 * Return Voice Type.
	 *
	 * @param $lang_name - Google voice name.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 **/
	public function get_voice_type( $lang_name ) {

		$pos = strpos( $lang_name, 'Wavenet' );

		if ( false === $pos ) {
			return '<img src="' . Voicer::$url . 'images/standard.svg" alt="Standard">' . esc_html( 'Standard' );
		}

		return '<img src="' . Voicer::$url . 'images/wavenet.svg" alt="WaveNet">' . esc_html( 'WaveNet' );

	}

	/**
	 * Return language name by code.
	 *
	 * @param $lang_code - Google language code.
	 *
	 * @return mixed
	 * @since 2.0.0
	 * @access public
	 */
	public function get_lang_by_code( $lang_code ) {

		if ( is_object( $lang_code ) ) {
			$lang_code = $lang_code[0];
		}

		/** @noinspection SpellCheckingInspection */
		$languages = [
			'da-DK'  => 'Danish (Dansk)',
			'nl-NL'  => 'Dutch (Nederlands)',
			'en-AU'  => 'English (Australian)',
			'en-GB'  => 'English (UK)',
			'en-US'  => 'English (US)',
			'fr-CA'  => 'French Canada (Français)',
			'fr-FR'  => 'French France (Français)',
			'de-DE'  => 'German (Deutsch)',
			'it-IT'  => 'Italian (Italiano)',
			'ja-JP'  => 'Japanese (日本語)',
			'ko-KR'  => 'Korean (한국어)',
			'nb-NO'  => 'Norwegian (Norsk)',
			'pl-PL'  => 'Polish (Polski)',
			'pt-BR'  => 'Portuguese Brazil (Português)',
			'pt-PT'  => 'Portuguese Portugal (Portugal)',
			'ru-RU'  => 'Russian (Русский)',
			'sk-SK'  => 'Slovak (Slovenčina)',
			'es-ES'  => 'Spanish (Español)',
			'sv-SE'  => 'Swedish (Svenska)',
			'tr-TR'  => 'Turkish (Türkçe)',
			'uk-UA'  => 'Ukrainian (Українська)',
			'ar-XA'  => 'Arabic (العربية)',
			'cs-CZ'  => 'Czech (Čeština)',
			'el-GR'  => 'Greek (Ελληνικά)',
			'en-IN'  => 'Indian English',
			'fi-FI'  => 'Finnish (Suomi)',
			'vi-VN'  => 'Vietnamese (Tiếng Việt)',
			'id-ID'  => 'Indonesian (Bahasa Indonesia)',
			'fil-PH' => 'Philippines (Filipino)',
			'hi-IN'  => 'Hindi (हिन्दी)',
			'hu-HU'  => 'Hungarian (Magyar)',
			'cmn-CN' => 'Chinese (官话)',
            'cmn-TW' => 'Taiwanese Mandarin (中文(台灣))',
            'bn-IN' => 'Bengali (বাংলা)',
            'gu-IN' => 'Gujarati (ગુજરાતી)',
            'kn-IN' => 'Kannada (ಕನ್ನಡ)',
            'ml-IN' => 'Malayalam (മലയാളം)',
            'ta-IN' => 'Tamil (தமிழ்)',
            'te-IN' => 'Telugu (తెలుగు)',
			'th-TH'  => 'Thai (ภาษาไทย)',
		];

		return $languages[ $lang_code ];
	}

	/** @noinspection PhpUnused */
	/**
	 * Render Drag & Drop API Key field.
	 *
	 * @since 3.0.0
	 * @access public
	 **/
	public static function render_dnd_api_key() {

		$key_exist = false;
		if ( self::get_instance()->options['dnd-api-key'] ) { $key_exist = true; }

		?>
        <div class="mdp-dnd">
            <!--suppress HtmlFormInputWithoutLabel -->
            <div class="mdc-text-field mdc-input-width mdc-text-field--outlined mdc-hidden">
                <!--suppress HtmlFormInputWithoutLabel -->
                <input  type="text"
                        class="mdc-text-field__input"
                        name="mdp_voicer_settings[dnd-api-key]"
                        id="mdp-voicer-settings-dnd-api-key"
                        value="<?php esc_attr_e( self::get_instance()->options['dnd-api-key'] ); ?>"
                >
                <div class="mdc-notched-outline mdc-notched-outline--upgraded mdc-notched-outline--notched">
                    <div class="mdc-notched-outline__leading"></div>
                    <div class="mdc-notched-outline__notch">
                        <label for="mdp-voicer-settings-dnd-api-key" class="mdc-floating-label mdc-floating-label--float-above"><?php esc_html_e( 'API Key', 'voicer' ); ?></label>
                    </div>
                    <div class="mdc-notched-outline__trailing"></div>
                </div>
            </div>
            <div id="mdp-api-key-drop-zone" class="<?php if ( $key_exist ) : ?>mdp-key-uploaded<?php endif; ?>">
				<?php if ( $key_exist ) : ?>
                    <span class="material-icons">check_circle_outline</span><?php esc_html_e( 'API Key file exist', 'voicer' ); ?>
                    <span class="mdp-drop-zone-hover"><?php esc_html_e( 'Drop Key file here or click to upload', 'voicer' ); ?></span>
				<?php else : ?>
                    <span class="material-icons">cloud</span><?php esc_html_e( 'Drop Key file here or click to upload.', 'voicer' ); ?>
				<?php endif; ?>
            </div>
			<?php if ( $key_exist ) : ?>
                <div class="mdp-messages mdc-text-field-helper-line mdc-text-field-helper-text mdc-text-field-helper-text--persistent">
					<?php esc_html_e( 'Drag and drop or click on the form to replace API key. |', 'voicer' ); ?>
                    <a href="#" class="mdp-reset-key-btn"><?php esc_html_e( 'Reset API Key', 'voicer' ); ?></a>
                </div>
			<?php else : ?>
                <div class="mdp-messages mdc-text-field-helper-line mdc-text-field-helper-text mdc-text-field-helper-text--persistent">
					<?php esc_html_e( 'Drag and drop or click on the form to add API key.', 'voicer' ); ?>
                </div>
			<?php endif; ?>
            <input id="mdp-dnd-file-input" type="file" name="name" class="mdc-hidden" />
        </div>
		<?php

	}

	/** @noinspection PhpUnused */
	/**
	 * Render settings current_language field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
    public function render_current_language() {
	    ?>
        <div class="mdp-now-used">
            <div>
                <strong><?php echo esc_attr( $this->options['language'] ); ?></strong>
            </div>
            <div>
                <audio controls="">
                    <source src="https://cloud.google.com/text-to-speech/docs/audio/<?php esc_attr_e( self::$instance->options['language'] ); ?>.mp3" type="audio/mp3">
				    <?php esc_html_e( 'Your browser does not support the audio element.', 'voicer' ); ?>
                </audio>
            </div>
        </div>
	    <?php
    }

	/** @noinspection PhpUnused */
	/**
	 * Render Tooltip Background Color field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_bgcolor() {

		/** Render colorpicker. */
		UI::get_instance()->render_colorpicker(
			$this->options['bgcolor'],
			esc_html__( 'Background Color', 'voicer' ),
			esc_html__( 'Select tooltip background-color', 'voicer' ),
			[
				'name' => 'mdp_voicer_design_settings[bgcolor]',
				'id' => 'mdp-voicer-bgcolor',
				'readonly' => 'readonly'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Icon Color field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_iconcolor() {

		/** Render colorpicker. */
		UI::get_instance()->render_colorpicker(
			$this->options['iconcolor'],
			esc_html__( 'Icon Color', 'voicer' ),
			esc_html__( 'Select icon color', 'voicer' ),
			[
				'name' => 'mdp_voicer_design_settings[iconcolor]',
				'id' => 'mdp-voicer-iconcolor',
				'readonly' => 'readonly'
			]
		);

	}

	/**
	 * Add plugin settings page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function add_settings_page() {

		add_action( 'admin_menu', [ $this, 'add_admin_menu' ] );
		add_action( 'admin_init', [ $this, 'settings_init' ] );

	}

	/**
	 * Call render_$key for each field.
	 *
	 * @param $group_name - Settings group name.
	 * @param $option_name - Settings option name.
	 * @param $section_id - Settings section id.
	 * @param array $fields - Settings fields [ key => label ] to render.
	 *
	 * @since 2.0.0
	 * @access public
	 */
	public function call_render_handlers( $group_name, $option_name, $section_id, $fields ) {

	    /** Create settings section. */
		register_setting( $group_name, $option_name );
		add_settings_section( $section_id, '', null, $group_name );

		/** Call render_$key for each field. */
		foreach ( $fields as $key => $label ) {

			/** Skip if render method not exist. */
			if ( ! is_callable( [$this, 'render_' . $key] ) ) { continue; }

			add_settings_field( $key, $label, [$this, 'render_' . $key ], $group_name, $section_id );
		}

    }

	/**
	 * Create Design Tab.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function design_tab() {

		/** Design Tab. */
		$group_name = 'DesignOptionsGroup';
		$section_id = 'mdp_voicer_settings_page_design_section';

		/** Settings fields on general tab. */
		$fields = [];

        $fields['bgcolor'] = esc_html__( 'Background Color:', 'voicer' );
        $fields['iconcolor'] = esc_html__( 'Icon Color:', 'voicer' );

		/** Call render_$key for each field. */
		$this->call_render_handlers( $group_name, 'mdp_voicer_design_settings', $section_id, $fields );

	}

	/**
	 * Create General Tab.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
    public function general_tab() {

	    /** General Tab. */
	    $group_name = 'VoicerOptionsGroup';
	    $section_id = 'mdp_voicer_settings_page_general_section';

	    /** Settings fields on general tab. */
	    $fields = [];

	    /** This settings available only after api key */
	    if ( self::$instance->options['dnd-api-key'] ) {

		    $fields['current_language'] = esc_html__( 'Now used:', 'voicer' );
		    $fields['language'] = esc_html__( 'Select Language:', 'voicer' );
		    $fields['audio_profile'] = esc_html__( 'Audio Profile:', 'voicer' );
		    $fields['speaking_rate'] = esc_html__( 'Speaking Speed:', 'voicer' );
		    $fields['pitch'] = esc_html__( 'Pitch:', 'voicer' );

        }

	    $fields ['dnd_api_key'] = esc_html__( 'API Key File:', 'voicer' );

	    /** Call render_$key for each field. */
	    $this->call_render_handlers( $group_name, 'mdp_voicer_settings', $section_id, $fields );

    }

	/**
	 * Generate Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function settings_init() {

		/** General Tab. */
	    $this->general_tab();

		/** This tab available only after api key. */
		if ( self::$instance->options['dnd-api-key'] ) {

			/** Design Tab. */
			$this->design_tab();

			/** Assignments Tab. */
			AssignmentsTab::get_instance()->add_settings();

        }

		/** Activation Tab. */
		PluginActivation::get_instance()->add_settings();

		/** Status Tab. */
		StatusTab::get_instance()->add_settings();

		/** Uninstall Tab. */
		UninstallTab::get_instance()->add_settings();

	}

	/**
	 * Add admin menu for plugin settings.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function add_admin_menu() {

		add_menu_page(
			esc_html__( 'Voicer Settings', 'voicer' ),
			esc_html__( 'Voicer', 'voicer' ),
			'manage_options',
			'mdp_voicer_settings',
			[ $this, 'options_page' ],
			'data:image/svg+xml;base64,' . base64_encode( file_get_contents( Voicer::$path . 'images/logo-menu.svg' ) ),
			'58.5994'// Always change digits after "." for different plugins.
		);

	}

	/**
	 * Plugin Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function options_page() {

		/** User rights check. */
		if ( ! current_user_can( 'manage_options' ) ) { return; } ?>

        <!--suppress HtmlUnknownTarget -->
        <form action='options.php' method='post'>
            <div class="wrap">

				<?php
				/** Get active tab slug. */
				$tab = $this->get_active_tab();

				/** Render "Voicer settings saved!" message. */
				$this->render_nags();

	            /** Render Tabs Headers. */
	            ?><section class="mdp-aside"><?php $this->print_tabs( $tab ); ?></section><?php

				/** Render Tabs Body. */
				?><section class="mdp-tab-content"><?php

					/** General Tab. */
					if ( 'general' === $tab ) {

						echo '<h3>' . esc_html__( 'Voicer Settings', 'voicer' ) . '</h3>';
						settings_fields( 'VoicerOptionsGroup' );
						do_settings_sections( 'VoicerOptionsGroup' );

                    /** Design Tab. */
					} elseif ( 'design' === $tab ) {

						echo '<h3>' . esc_html__( 'Design Settings', 'voicer' ) . '</h3>';
						settings_fields( 'DesignOptionsGroup' );
						do_settings_sections( 'DesignOptionsGroup' );

                    /** Assignments Tab. */
					} elseif ( 'assignments' === $tab ) {

						echo '<h3>' . esc_html__( 'Assignments Settings', 'voicer' ) . '</h3>';
						settings_fields( 'VoicerAssignmentsOptionsGroup' );
						do_settings_sections( 'VoicerAssignmentsOptionsGroup' );
						AssignmentsTab::get_instance()->render_assignments();

                    /** Activation Tab. */
					} elseif ( 'activation' === $tab ) {

						settings_fields( 'VoicerActivationOptionsGroup' );
						do_settings_sections( 'VoicerActivationOptionsGroup' );
						PluginActivation::get_instance()->render_pid();

                    /** Status tab. */
					} elseif ( 'status' === $tab ) {

						echo '<h3>' . esc_html__( 'System Requirements', 'voicer' ) . '</h3>';
						StatusTab::get_instance()->render_form();

                    /** Uninstall Tab. */
					} elseif ( 'uninstall' === $tab ) {

						echo '<h3>' . esc_html__( 'Uninstall Settings', 'voicer' ) . '</h3>';
						UninstallTab::get_instance()->render_form();

					}

					?>
                </section>
            </div>
        </form>

		<?php
	}

	/**
	 * Render Tabs Headers.
	 *
	 * @param string $current - Selected tab key.
	 *
	 * @since 2.1.0
	 * @access public
	 **/
	public function print_tabs( $current ) {

		/** Get available tabs. */
		$tabs = $this->get_tabs();

		/** Render Tabs. */
		?>
        <aside class="mdc-drawer">
            <div class="mdc-drawer__content">
                <nav class="mdc-list">
					<?php

					/** Render logo in plugin settings. */
					$this->render_logo();

					/** Render settings tabs. */
					$this->render_tabs( $tabs, $current );

					/** Helpful links. */
					$this->support_link();

					/** Activation Status. */
					PluginActivation::get_instance()->display_status();

					?>
                </nav>
            </div>
        </aside>
		<?php
	}

	/**
	 * Render logo and Save changes button in plugin settings.
	 *
	 * @access private
	 * @since 3.0.0
	 *
	 * @return void
	 **/
	private function render_logo() {

		?>
        <div class="mdc-drawer__header mdc-plugin-fixed">
            <!--suppress HtmlUnknownAnchorTarget -->
            <a class="mdc-list-item mdp-plugin-title" href="#wpwrap">
                <i class="mdc-list-item__graphic" aria-hidden="true">
                    <img src="<?php echo esc_attr( Voicer::$url . 'images/logo-color.svg' ); ?>" alt="<?php echo esc_html__( 'Voicer', 'voicer' ) ?>">
                </i>
                <span class="mdc-list-item__text">
                    <?php echo esc_html__( 'Voicer', 'voicer' ) ?>
                    <sup><?php echo esc_html__( 'ver.', 'voicer' ) . esc_html( Voicer::$version ); ?></sup>
                </span>
            </a>
            <button type="submit" name="submit" id="submit" class="mdc-button mdc-button--dense mdc-button--raised">
                <span class="mdc-button__label"><?php echo esc_html__( 'Save changes', 'voicer' ) ?></span>
            </button>
        </div>
		<?php

	}

	/**
	 * Return an array of available tabs in plugin settings.
	 *
	 * @access private
	 * @since 3.0.0
	 *
	 * @return array
	 **/
	private function get_tabs() {

		$tabs = [];

		/** Check for required extensions. */
		if ( CheckCompatibility::get_instance()->do_settings_checks( true ) ) {

			$tabs['general'] = [
				'icon' => 'tune',
				'name' => esc_html__( 'General', 'voicer' )
			];

			/** Adds key dependent tabs: design, css, assignments. */
			$tabs = $this->add_key_dependent_tabs( $tabs );

		}

		/** Adds activation tab. */
		$tabs = $this->add_activation_tab( $tabs );

		$tabs['status'] = [
			'icon' => 'info',
			'name' => esc_html__( 'Status', 'voicer' )
		];

		$tabs['uninstall'] = [
			'icon' => 'delete_sweep',
			'name' => esc_html__( 'Uninstall', 'voicer' )
		];

		return $tabs;

	}

	/**
	 * Adds activation tab if plugin published on CodeCanyon.
	 *
	 * @param array $tabs - Array of tabs to show in plugin settings.
	 *
	 * @access private
	 * @since  2.1.0
	 *
	 * @return array - Array of tabs to show in plugin settings.
	 **/
	private function add_activation_tab( $tabs ) {

		/** Enable Activation tab only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();

		if ( (int)$plugin_id > 0 ) {

			$tabs['activation'] = [
				'icon' => 'vpn_key',
				'name' => esc_html__( 'Activation', 'voicer' )
			];

		}

		return $tabs;

	}

	/**
	 * Adds key dependent tabs: design, css, assignments.
	 *
	 * @param array $tabs - Array of tabs to show in plugin settings.
	 *
	 * @access private
	 * @since  2.1.0
	 *
	 * @return array - Array of tabs to show in plugin settings.
	 **/
	private function add_key_dependent_tabs( $tabs ) {

		/** Show this tabs only if we have key file. */
		if ( ! $this->options['dnd-api-key'] ) { return $tabs; }

		$tabs['design'] = [
			'icon' => 'brush',
			'name' => esc_html__( 'Design', 'voicer' )
		];

		$tabs['assignments'] = [
			'icon' => 'flag',
			'name' => esc_html__( 'Assignments', 'voicer' )
		];

		return $tabs;

	}

	/**
	 * Return active tab slug.
	 *
	 * @access private
	 * @since 2.1.0
	 *
	 * @return void
	 **/
	private function get_active_tab() {

		/** Default Tab. */
		$tab = 'status';

		/** If the min requirements are met, show the general tab. */
		$min_requirements = CheckCompatibility::get_instance()->do_settings_checks( false );
		if ( $min_requirements ) {
			$tab = 'general';
		}

		/** Tab selected by user. */
		if ( isset ( $_GET['tab'] ) ) {
			$tab = filter_input( INPUT_GET, 'tab', FILTER_SANITIZE_STRING );
		}

		/** If the min requirements are not met and selected hidden tab - show the status tab. */
		if ( ( ! $min_requirements ) && in_array( $tab, ['general', 'design', 'assignments'], true ) ) {
			$tab = 'status';
		}

		return $tab;

	}

	/**
	 * Render "Settings Saved" nags.
	 *
	 * @since 2.0.0
     *
     * @return void
	 **/
	public function render_nags() {

		if ( ! isset( $_GET['settings-updated'] ) ) { return; }

		if ( strcmp( $_GET['settings-updated'], "true" ) === 0 ) {

			/** Render "Settings Saved" message. */
			UI::get_instance()->render_snackbar( esc_html__( 'Settings saved!', 'voicer' ) );

		}

		if ( ! isset( $_GET['tab'] ) ) { return; }

		if ( strcmp( $_GET['tab'], "activation" ) === 0 ) {

			if ( PluginActivation::get_instance()->is_activated() ) {

				/** Render "Activation success" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Plugin activated successfully.', 'voicer' ), 'success', 5500 );

			} else {

				/** Render "Activation failed" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Invalid purchase code.', 'voicer' ), 'error', 5500 );

			}

		}

	}

	/**
	 * Get plugin settings with default values.
	 *
	 * @since 2.0.0
	 * @access public
	 * @return void
	 **/
	public function get_options() {

		/** Default values. */
		$defaults = [
			// TODO: Remove 'api-key-file' in 2.1.1
			'api-key-file'  => '', // API Key file.
			'dnd-api-key'   => '',  // Encoded JSON API Key file.
			'language'      => 'en-US-Standard-D', // Language.
			'language-code' => 'en-US', // Language Code.
			'audio-profile' => 'handset-class-device', // Audio profile.
			'speaking-rate' => '1', // Speaking rate/speed.
			'pitch'         => '0', // Pitch.

			# Design Tab.
			'bgcolor'       => '#0253ee', // Tooltip background color.
			'iconcolor'     => '#fff', // Icon color.
		];

		/** Get Options from WP. */
		$options = get_option( 'mdp_voicer_settings' );
		$results = wp_parse_args( $options, $defaults );

		// TODO: Remove migrate_key_file_to_settings() in 2.1.0
		/** Moving from uploaded key file to stored in DB value. */
		$results = $this->migrate_key_file_to_settings( $results );

		/** Design tab settings. */
		$design_settings = get_option( 'mdp_voicer_design_settings' );
		$results = wp_parse_args( $design_settings, $results );

		/** Reset API Key on fatal error. */
		if ( isset( $_GET['reset-api-key'] ) && '1' === $_GET['reset-api-key'] ) {

			$this->reset_api_key();

		}

		$this->options = $results;
	}

	/**
	 * Reset API Key on fatal error.
	 *
	 * @since  2.1.0
	 * @access public
	 * @return void
	 **/
	private function reset_api_key() {

		/** Remove API Key. */
		$options = get_option( 'mdp_voicer_settings' );
		$options['dnd-api-key'] = '';

		/** Save new value. */
		update_option( 'mdp_voicer_settings', $options );

		/** Go to first tab. */
		wp_redirect( admin_url( '/admin.php?page=mdp_voicer_settings&tab=general' ) );
		exit;

	}

	/**
	 * Moving from uploaded key file to stored in DB value.
	 *
	 * @param array $options - All plugin settings.
	 *
	 * @since  2.1.0
	 * @access public
	 * @return void|array
	 **/
	private function migrate_key_file_to_settings( $options ) {

		/** Do nothing if we already have dnd-api-key value. */
		if ( isset( $options['dnd-api-key'] ) && strlen( $options['dnd-api-key'] ) > 10 ) { return $options; }

		/** Do we have key file? */
		if ( ! isset( $options['api-key-file'] ) ) { return $options; }

		/** File ID is empty. */
		if ( ! trim( $options['api-key-file'] ) ) {  return $options; }

		/** Get path to file by attachments id. */
		$key_path = get_attached_file( $options['api-key-file'] );

		/** The WordPress filesystem. */
		global $wp_filesystem;

		/** Instantiate the WordPress filesystem. */
		Helper::init_filesystem();

		/** Read Key File Content. */
		$key_content = $wp_filesystem->get_contents( $key_path );

		/** Check if json is valid for the key. */
		if ( is_object( json_decode( $key_content, false ) ) ) {

			/** Convert old value to new format. */
			$options['dnd-api-key'] =  base64_encode( $key_content );

		} else {

			/** Clear dnd-api-key option. */
			$options['dnd-api-key'] = '';

		}

		/** Clear Old api-key-file option. */
		$options['api-key-file'] = '';

		/** Update new values in DB. */
		update_option( 'mdp_voicer_settings', $options );

		return $options;

	}

	/**
	 * Main Settings Instance.
	 *
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
	 * @return Settings
	 * @since 2.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

} // End Class Settings.
