<?php
/**
 * Plugin Name: Voicer
 * Plugin URI: https://1.envato.market/voicer
 * Description: Voicer converts text into human-like speech in more than 200 voices across 30+ languages and variants.
 * Author: 20script
 * Version: 2.1.3
 * Author URI: https://www.20script.ir
 * Requires PHP: 5.6
 * Requires at least: 3.0
 * Tested up to: 5.5
 **/

/**
 * Voicer converts text into human-like speech in more than 200 voices across 30+ languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/voicer
 *
 * @encoding        UTF-8
 * @version         2.1.3
 * @copyright       Copyright (C) 2018 - 2020 Merkulove (https://merkulov.design/). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

/** Include plugin autoloader for additional classes. */
require __DIR__ . '/src/autoload.php';

use DOMXPath;
use DOMDocument;
use Merkulove\Voicer\ErrorHandler;
use Merkulove\Voicer\UI;
use Merkulove\Voicer\Helper;
use Merkulove\Voicer\Settings;
use Merkulove\Voicer\EnvatoItem;
use Merkulove\Voicer\Shortcodes;
use Merkulove\Voicer\PluginHelper;
use Merkulove\Voicer\PluginUpdater;
use Merkulove\Voicer\AssignmentsTab;
use Merkulove\Voicer\simple_html_dom;
use Merkulove\Voicer\CheckCompatibility;

/** Includes the autoloader for libraries installed with Composer. */
require __DIR__ . '/vendor/autoload.php';

/** Imports the Cloud Client Library. */
use Merkulove\Voicer\XMLHelper;
use Google\ApiCore\ApiException;
use Google\Cloud\TextToSpeech\V1\AudioConfig;
use Google\Cloud\TextToSpeech\V1\AudioEncoding;
use Google\Cloud\TextToSpeech\V1\SynthesisInput;
use Google\Cloud\TextToSpeech\V1\TextToSpeechClient;
use Google\Cloud\TextToSpeech\V1\VoiceSelectionParams;

/**
 * SINGLETON: Core class used to implement a Voicer plugin.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * @since 1.0.0
 **/
final class Voicer {

	/**
	 * Plugin version.
	 *
	 * @string version
	 * @since 2.0.0
	 **/
	public static $version = '';

	/**
	 * Use minified libraries if SCRIPT_DEBUG is turned off.
	 *
	 * @since 2.0.0
	 **/
	public static $suffix = '';

	/**
	 * URL (with trailing slash) to plugin folder.
	 *
	 * @var string
	 * @since 2.0.0
	 **/
	public static $url = '';

	/**
	 * PATH to plugin folder.
	 *
	 * @var string
	 * @since 2.0.0
	 **/
	public static $path = '';

	/**
	 * Plugin base name.
	 *
	 * @var string
	 * @since 2.0.0
	 **/
	public static $basename = '';

	/**
	 * Plugin admin menu base.
	 *
	 * @var string
	 * @since 2.1.0
	 **/
	public static $menu_base;

    /**
     * The one true Voicer.
     *
     * @var Voicer
     * @since 1.0.0
     **/
    private static $instance;

    /**
     * Sets up a new plugin instance.
     *
     * @since 1.0.0
     * @access public
     **/
    private function __construct() {

	    /** Initialize main variables. */
	    $this->initialization();

    }

	/**
	 * Setup the plugin.
	 *
	 * @since 2.1.0
	 * @access public
	 *
	 * @return void
	 **/
	public function setup() {

		/** Do critical initial checks. */
		if ( ! CheckCompatibility::get_instance()->do_initial_checks( true ) ) { return; }

		/** Send install Action to our host. */
		self::send_install_action();

		/** Define hooks that runs on both the front-end as well as the dashboard. */
		$this->both_hooks();

		/** Define admin hooks. */
		$this->admin_hooks();

		/** Define public hooks. */
		$this->public_hooks();

    }

	/**
	 * Send install Action to our host.
	 *
	 * @static
	 * @since 2.1.0
	 **/
	private static function send_install_action() {

		/** Plugin version. */
		$ver = self::get_instance()->get_version();

		/** Have we already sent 'install' for this version? */
		$opt_name = 'mdp_voicer_send_action_install';
		$ver_installed = get_option( $opt_name );

		/** Send install Action to our host. */
		if ( ! $ver_installed || $ver !== $ver_installed ) {

			/** Send install Action to our host. */
			Helper::get_instance()->send_action( 'install', 'voicer', $ver );
			update_option( $opt_name, $ver );

		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function public_hooks() {

		/** Add plugin script. */
		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ] );

		/** Ajax on the Viewer-Facing Side. */
		add_action( 'wp_ajax_voicergspeak', [ $this, 'gspeak' ] );
		add_action( 'wp_ajax_nopriv_voicergspeak', [ $this, 'gspeak' ] );

    }

	/**
	 * Define hooks that runs on both the front-end as well as the dashboard.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	private function both_hooks() {

		/** Load translation. */
		$this->load_textdomain();

		/** Initialize plugin settings. */
		Settings::get_instance();

		/** Plugin update mechanism enable only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();
		if ( (int)$plugin_id > 0 ) {
			PluginUpdater::get_instance();
		}

		/** Adds all the necessary shortcodes. */
		Shortcodes::get_instance();

    }

	/**
	 * Register all of the hooks related to the admin area functionality.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	private function admin_hooks() {

		/** Initialize PluginHelper. */
		PluginHelper::get_instance();

		/** Add plugin settings page. */
		Settings::get_instance()->add_settings_page();

		/** Load JS and CSS for Backend Area. */
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_scripts' ] );

		/** Allow JSON files in the media library. */
		add_filter( 'upload_mimes', [ $this, 'allow_json_uploads' ], 1, 1 );

		/** Remove all "third-party" notices from plugin settings page. */
		add_action( 'in_admin_header', [$this, 'remove_all_notices'], 1000 );

		/** Show admin warning, if we need API Key. */
        if( ! Settings::get_instance()->options['dnd-api-key'] ){
            add_action( 'admin_notices', [ $this, 'key_notice' ] );
        }

		/** Remove "Thank you for creating with WordPress" and WP version only from plugin settings page. */
		add_action( 'admin_enqueue_scripts', [$this, 'remove_wp_copyrights'] );

    }

	/**
	 * Remove all other notices.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function remove_all_notices() {

		/** Work only on plugin settings page. */
		$screen = get_current_screen();
		if ( null === $screen ) { return; }
		if ( self::$menu_base !== $screen->base ) { return; }

		/** Remove other notices. */
		remove_all_actions( 'admin_notices' );
		remove_all_actions( 'all_admin_notices' );

		/** Show admin warning, if we need API Key. */
		if ( ! Settings::get_instance()->options['dnd-api-key'] ) {
			add_action( 'admin_notices', [ $this, 'key_notice' ] );
		}

	}

	/**
	 * Initialize main variables.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function initialization() {

		/** Get Plugin version. */
		$plugin_data = $this->get_plugin_data();
		self::$version = $plugin_data['Version'];

		/** Gets the plugin URL (with trailing slash). */
		self::$url = plugin_dir_url( __FILE__ );

		/** Gets the plugin PATH. */
		self::$path = plugin_dir_path( __FILE__ );

		/** Use minified libraries if SCRIPT_DEBUG is turned off. */
		self::$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		/** Set plugin basename. */
		self::$basename = plugin_basename( __FILE__ );

		/** Plugin settings page base. */
		self::$menu_base = 'toplevel_page_mdp_voicer_settings';

	}

	/**
	 * Return plugin version.
	 *
	 * @return string
	 * @since 2.0.2
	 * @access public
	 **/
	public function get_version() {

		return self::$version;

	}

	/**
	 * Return current plugin metadata.
	 *
	 * @since 2.1.0
	 * @access public
	 *
	 * @return array {
	 *     Plugin data. Values will be empty if not supplied by the plugin.
	 *
	 *     @type string $Name        Name of the plugin. Should be unique.
	 *     @type string $Title       Title of the plugin and link to the plugin's site (if set).
	 *     @type string $Description Plugin description.
	 *     @type string $Author      Author's name.
	 *     @type string $AuthorURI   Author's website address (if set).
	 *     @type string $Version     Plugin version.
	 *     @type string $TextDomain  Plugin textdomain.
	 *     @type string $DomainPath  Plugins relative directory path to .mo files.
	 *     @type bool   $Network     Whether the plugin can only be activated network-wide.
	 *     @type string $RequiresWP  Minimum required version of WordPress.
	 *     @type string $RequiresPHP Minimum required version of PHP.
	 * }
	 **/
	public function get_plugin_data() {

		if ( ! function_exists('get_plugin_data') ) {
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		return get_plugin_data( __FILE__ );

	}

	/**
	 * Ajax front-end action hook here.
	 *
	 * @throws ApiException
	 * @since 1.0.0
	 * @access public
     *
     * @return void
	 **/
    public function gspeak() {

	    /** Security Check. */
	    check_ajax_referer( 'voicer-nonce', 'nonce' );

	    /** Setting custom exception handler. */
	    set_exception_handler( [ ErrorHandler::class, 'exception_handler' ] );

        /** Instantiates a client. */
        $client = new TextToSpeechClient();

        /** Sets text to be synthesised. */
        $html = filter_input( INPUT_POST, 'text' );

	    /** Remove muted elements by class "voicer-mute" or attribute voicer-mute="". */
	    $html = $this->remove_muted_html( $html );

	    /**  Replace <span voicer-break=""></span> to <break time="200ms"/>. */
	    $html = $this->replace_break_tag( $html );

	    /** Clean HTML. */
	    $html = XMLHelper::get_instance()->clean_html( $html );

	    /** Strip all html tags, except SSML tags.  */
	    $html = strip_tags( $html, '<p><break><say-as><sub><emphasis>');

	    /** Remove the white spaces from the left and right sides.  */
	    $html = trim( $html );

	    /** Convert HTML entities to their corresponding characters: &quot; => " */
	    $html = html_entity_decode( $html );

	    /** Force to SSML. */
	    $ssml = "<speak>";
	    $ssml .= $html;
	    $ssml .= "</speak>";

	    /** Sets text to be synthesised. */
	    $synthesisInputText = ( new SynthesisInput() )->setSsml( $ssml );

	    /** Get plugin settings. */
	    $options = Settings::get_instance()->options;

        /** Build the voice request, select the language. */
        $voice = (new VoiceSelectionParams())
            ->setLanguageCode( $options['language-code'] )
            ->setName( $options['language'] );

        /** Effects profile. */
        $effectsProfileId = $options['audio-profile'];

        /** Select the type of audio file you want returned. */
        $audioConfig = ( new AudioConfig() )
                ->setAudioEncoding( AudioEncoding::MP3 )
                ->setEffectsProfileId( [ $effectsProfileId ] )
                ->setSpeakingRate( $options['speaking-rate'] )
                ->setPitch( $options['pitch'] )
	            ->setSampleRateHertz( 24000 );


        /** Perform text-to-speech request on the text input with selected voice. */
        $response = $client->synthesizeSpeech( $synthesisInputText, $voice, $audioConfig );
        $audioContent = $response->getAudioContent();

	    /** Restore previous exception handler. */
	    restore_exception_handler();

        /** The response's audioContent is binary. */
        echo $audioContent;

        wp_die();

    }

	/**
	 * Replace <span voicer-break=""></span> to <break time="200ms"/>.
	 *
	 * @param $html_content string - HTML code.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 */
	public function replace_break_tag( $html_content ) {

		/** Create a DOM object. */
		$html = new simple_html_dom();

		/** Load HTML from a string. */
		$html->load( $html_content );

		/** Foreach element wit attribute voicer-break="". */
		foreach ( $html->find( '[voicer-break]' ) as $el ) {

			/** Do replacements. */
			$break_tag = $el->outertext;
			$break_tag = str_replace( '<span voicer-break=""', '<break', $break_tag );

			$el->outertext = $break_tag;
		}

		/** Return result. */
		return $html->save();

	}

	/**
	 * Remove muted elements by class "voicer-mute" or attribute voicer-mute="".
	 *
	 * @param $post_content - Post/Page content.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 *
	 */
	public function remove_muted_html( $post_content ) {

		/** Hide DOM parsing errors. */
		libxml_use_internal_errors( true );
		libxml_clear_errors();

		/** Load the possibly malformed HTML into a DOMDocument. */
		$dom          = new DOMDocument();
		$dom->recover = true;
		$dom->loadHTML( '<?xml encoding="UTF-8"><body id="repair">' . $post_content . '</body>' ); // input UTF-8.

		$selector = new DOMXPath( $dom );

		/** Remove all elements with voicer-mute="" attribute. */
		foreach( $selector->query( '//*[@voicer-mute]') as $e ) {
			$e->parentNode->removeChild( $e );
		}

		/** Remove all elements with class="voicer-mute". */
		foreach( $selector->query( '//*[contains(attribute::class, "voicer-mute")]' ) as $e ) {
			$e->parentNode->removeChild( $e );
		}

		/** HTML without muted tags. */
		$body = $dom->documentElement->lastChild;

		return trim( XMLHelper::get_instance()->get_inner_html( $body ) );
	}

	/**
	 * Allow JSON files in the media library.
	 *
	 * @param $mime_types - Current array of mime types.
	 *
	 * @return array - Updated array of mime types.
	 * @since 1.0.0
	 * @access public
	 **/
    public function allow_json_uploads( $mime_types ) {

        /** Adding .json extension. */
        $mime_types['json'] = 'text/plain';

        return $mime_types;
    }

	/**
	 * Show admin warning, if we need API Key.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function key_notice() {

		/** Get current screen. */
		$screen = get_current_screen();
		if ( null === $screen ) { return; }

		/** Voicer Settings Page. */
		if ( self::$menu_base === $screen->base  ) {

			/** Render "Before you start" message. */
			UI::get_instance()->render_snackbar(
				esc_html__( 'This plugin uses the Google Cloud Text-to-Speech API Key File. Set up your Google Cloud Platform project before the start.', 'voicer' ),
				'warning', // Type
				-1, // Timeout
				true, // Is Closable
				[ [ 'caption' => 'Get Key File', 'link' => 'https://docs.merkulov.design/about-key-file-for-the-voicer-wordpress-plugin/' ] ] // Buttons
			);

		} else { ?>

            <div class="settings-error notice notice-warning">
                <p><strong><?php esc_html_e( 'Voicer: Before you begin', 'voicer' ); ?></strong></p>
                <p><?php esc_html_e( 'This plugin uses the Cloud Text-to-Speech API. You need to set up your Google Cloud Platform project and authorization before creating audio from text. Visit', 'voicer' ); ?>
                    <a href="https://docs.merkulov.design/about-key-file-for-the-voicer-wordpress-plugin/"
                       target="_blank"><?php esc_html_e( 'Online Documentation', 'voicer' ); ?></a> <?php esc_html_e( 'for more details.', 'voicer' ); ?>
                </p>
            </div>

			<?php

		}

	}

	/**
	 * Remove "Thank you for creating with WordPress" and WP version only from plugin settings page.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	public function remove_wp_copyrights() {

		/** Remove "Thank you for creating with WordPress" and WP version from plugin settings page. */
		$screen = get_current_screen(); // Get current screen.
		if ( null === $screen ) { return; }

		/** Plugin Settings Page. */
		if ( self::$menu_base === $screen->base ) {
			add_filter( 'admin_footer_text', '__return_empty_string', 11 );
			add_filter( 'update_footer', '__return_empty_string', 11 );
		}

	}

    /**
     * Loads the Voicer translated strings.
     *
     * @since 1.0.0
     * @access public
     **/
    public function load_textdomain() {

        load_plugin_textdomain( 'voicer', false, self::$path . '/languages/' );

    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since   1.0.0
     * @return void
     **/
    public function enqueue_scripts() {

	    /** Checks if plugin have API Key. */
	    if( ! Settings::get_instance()->options['dnd-api-key'] ){ return; }

	    /** Checks if plugin should work on this page. */
	    if ( ! AssignmentsTab::get_instance()->display() ) { return; }

        wp_enqueue_script( 'mdp-voicer', self::$url . 'js/voicer' . self::$suffix . '.js', [], self::$version, true );
        wp_localize_script( 'mdp-voicer', 'mdp_voicer', [
            'ajaxurl'   => admin_url( 'admin-ajax.php' ),
            'nonce'     => wp_create_nonce( 'voicer-nonce' ), // Nonce for security.
            'bgcolor'   => Settings::get_instance()->options['bgcolor'],
            'iconcolor' => Settings::get_instance()->options['iconcolor']
        ] );
    }

    /**
     * Add CSS for admin area.
     *
     * @since   1.0.0
     * @return void
     **/
    public function admin_styles() {

        /** Add styles only on setting page */
        $screen = get_current_screen();
        if ( null === $screen ) { return; }

	    /** Voicer Settings Page. */
	    if ( self::$menu_base === $screen->base ) {

		    wp_enqueue_style( 'merkulov-ui', self::$url . 'css/merkulov-ui.min.css', [], self::$version );
		    wp_enqueue_style( 'dataTables', self::$url . 'css/jquery.dataTables' . self::$suffix . '.css', [], self::$version );
		    wp_enqueue_style( 'mdp-voicer-admin', self::$url . 'css/admin' . self::$suffix . '.css', [], self::$version );


	    } elseif ( $screen->base === 'plugin-install' ) {

		    /** Styles only for our plugin. */
		    if ( isset( $_GET['plugin'] ) AND $_GET['plugin'] === 'voicer' ) {
			    wp_enqueue_style( 'mdp-voicer-plugin-install', self::$url . 'css/plugin-install' . self::$suffix . '.css', [], self::$version );
		    }

	    }

    }

    /**
     * Add JS for admin area.
     *
     * @since   1.0.0
     * @return void
     **/
    public function admin_scripts() {

	    /** Add styles only on setting page */
	    $screen = get_current_screen();
	    if ( null === $screen ) { return; }

	    /** Voicer Settings Page. */
	    if ( self::$menu_base === $screen->base ) {

		    wp_enqueue_script( 'merkulov-ui', self::$url . 'js/merkulov-ui' . self::$suffix . '.js', [], self::$version, true );
		    wp_enqueue_script( 'dataTables', self::$url . 'js/jquery.dataTables' . self::$suffix . '.js', [ 'jquery' ], self::$version, true );
		    wp_enqueue_script( 'mdp-voicer-admin', self::$url . 'js/admin' . self::$suffix . '.js', ['jquery', 'dataTables'], self::$version, true );

	    }

    }

	/**
	 * Run when the plugin is activated.
	 *
	 * @static
	 * @since 2.0.0
	 **/
	public static function on_activation() {

		/** Security checks. */
		if ( ! current_user_can( 'activate_plugins' ) ) { return; }

		/** We need to know plugin to activate it. */
		if ( ! isset( $_REQUEST['plugin'] ) ) { return; }

		/** Get plugin. */
		$plugin = filter_var( $_REQUEST['plugin'], FILTER_SANITIZE_STRING );

		/** Checks that a user was referred from admin page with the correct security nonce. */
		check_admin_referer( "activate-plugin_{$plugin}" );

		/** Do critical initial checks. */
		if ( ! CheckCompatibility::get_instance()->do_initial_checks( false ) ) { return; }

		/** Send install Action to our host. */
		self::send_install_action();

	}

	/**
	 * Main Voicer Instance.
	 *
	 * Insures that only one instance of Voicer exists in memory at any one time.
	 *
	 * @static
	 * @return Voicer
	 * @since 1.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

} // End Class Voicer.

/** Run when the plugin is activated. */
register_activation_hook( __FILE__, [Voicer::class, 'on_activation'] );

/** Run plugin class once after activated plugins have loaded. */
add_action( 'plugins_loaded', [Voicer::get_instance(), 'setup'] );
